@@ -10,6 +10,7 @@ use k8s_openapi::apimachinery;
 use kube::api;
 use kube::config;
 use kube::{Resource, ResourceExt};
+use serde::{de, ser};
 use serde_yaml as yaml;
 
 use super::*;
@@ -55,6 +56,54 @@ impl Kubectl {
             .await
             .map_err(from_kubeconfig_error)
     }
+
+    pub async fn ensure_global_k_is_installed<K>(&self, mut object: K) -> kube::Result<K>
+    where
+        K: kube::ResourceExt + ser::Serialize + de::DeserializeOwned + Clone + fmt::Debug,
+        <K as kube::Resource>::DynamicType: Default,
+    {
+        let name = object.name();
+        let objects = self.api::<K>();
+        if let Ok(existing) = objects.get(&name).await {
+            object.meta_mut().resource_version = existing.resource_version();
+        }
+
+        let pp = self.post_params();
+
+        let object = if object.resource_version().is_some() {
+            objects.replace(&name, &pp, &object).await?
+        } else {
+            objects.create(&pp, &object).await?
+        };
+
+        Ok(object)
+    }
+
+    pub async fn ensure_namespaced_k_is_installed<K>(
+        &self,
+        mut object: K,
+        namespace: &str,
+    ) -> kube::Result<K>
+    where
+        K: kube::Resource + ser::Serialize + de::DeserializeOwned + Clone + fmt::Debug,
+        <K as kube::Resource>::DynamicType: Default,
+    {
+        let name = object.name();
+        let objects = self.namespaced_api::<K>(namespace);
+        if let Ok(existing) = objects.get(&name).await {
+            object.meta_mut().resource_version = existing.resource_version();
+        }
+
+        let pp = self.post_params();
+
+        let object = if object.resource_version().is_some() {
+            objects.replace(&name, &pp, &object).await?
+        } else {
+            objects.create(&pp, &object).await?
+        };
+
+        Ok(object)
+    }
 }
 
 impl fmt::Debug for Kubectl {
